package modules::10_hostdef;

use POSIX qw(strftime);
require taghelpers;

sub new
{
    my $class = shift;
    my $self = {};
    bless $self, $class;
    return $self;
}

sub getName
{
    return "Host Definition";
}

my @hosttypes = ( fbsdjail, openvz, vmware, vserver, xenparavirt, kvm );

sub parseTags
{
	my ($self,$sparser) = @_;

	my %tags;

	# Tag Werte zusammensuchen und aufbereiten, ggf. gibt es Zusatztext
	$tags{'check.mon'} = taghelpers::ensureTag($sparser,"Check.mon");
	$tags{'check.mon.freitext'} = taghelpers::ensureTag($sparser,"Check.mon.Freitext");
	# ebenso aufbereiten, welche Services explizit deaktiviert sind
	$tags{'check.mon.no'} = taghelpers::ensureTag($sparser,"Check.mon.no");
	$tags{'check.mon.no.freitext'} = taghelpers::ensureTag($sparser,"Check.mon.no.Freitext");
        # und tags fuer check_mk aufbereiten
	$tags{'check.mon.mk'} = taghelpers::ensureTag($sparser,"Check.mon.MK");

	my $checkmon = $tags{'check.mon'};
        $checkmon =~ s/^\s+|\s+$//g ; # remove leading and trailing spaces
	if (($checkmon eq "") || ($checkmon eq "no") || 
		($checkmon eq "none") || ($checkmon eq "manual"))
	{
		if ($main::verbose)
		{
			print(STDERR "Old deactivation scheme in ".$sparser->getFilename()."\n");
			print(STDERR "Possibly activate monitoring or use Check.mon: never/\$YYYYMMDD\n");
		}
		$tags{'check.mon'} = "";
		return %tags;
	}
	elsif ($checkmon eq "never")
	{
		if (! $tags{'check.mon.freitext'})
		{
			print(STDERR "Error in ".$sparser->getFilename()."\n");
			print(STDERR "'Check.mon: never' without stating a reason in 'Check.mon.Freitext'.\n");
			print(STDERR "Please fill 'Check.mon.Freitext' appropriately.\n");
			exit(1);
		}
		elsif ($main::verbose)
		{
			print(STDERR "Monitoring permanently deactivated in ".$sparser->getFilename()."\n");
			print(STDERR "Freitext: '".$tags{'check.mon.freitext'}."'\n\n");
		}
		$tags{'check.mon'} = "";
		return %tags;
	}
	elsif ($checkmon =~ /^(\d{8})$/)
	{
		my $wakeup = $1;
		my $yyyymmdd = strftime "%Y%m%d", localtime;
		# sehr simpler Parser, so vermeiden wir Extra-Module zu benoetigen
		if ($yyyymmdd gt $wakeup)
		{
			print(STDERR "Warning for ".$sparser->getFilename()."\n");
			print(STDERR "'Check.mon: $wakeup' in the past, so monitoring ought to be activated now.\n");
			print(STDERR "Please contact whoever originally set this date (see rcshist) to fix this.\n\n");
		}
		elsif ($wakeup - $yyyymmdd >= 8000)
		{
			print(STDERR "Warning in ".$sparser->getFilename()."\n");
			print(STDERR "'Check.mon: $wakeup' is pretty far in the future.\n\n");
		}
		elsif ($main::verbose)
		{
			print(STDERR "Monitoring temporarily deactivated in ".$sparser->getFilename()."\n");
			print(STDERR "Until: '$wakeup'\n\n");
		}
		$tags{'check.mon'} = "";
		return %tags;
	}

	# Hosttype einlesen und validieren
	$tags{'hosttype'} = taghelpers::ensureTag($sparser,"JAIL.Type");	
	if (index(join("",@hosttypes),$tags{'hosttype'} ) == -1)
	{
		print(STDERR "Error in ".$sparser->getFilename()."\n");
		print(STDERR "JAIL.Type error: $tags{'hosttype'}\n");
		exit(1);
	}

	# Funktionstag einlesen:
	#
	$tags{'funktion'} = taghelpers::ensureTag($sparser,"Funktion");


	# Hostname einlesen
	$tags{'hostname'} = taghelpers::ensureTag($sparser,"Hostname");
        $tags{'hostname'} =~ s/^\s+|\s+$//g ; # remove leading and trailing spaces
	if ($tags{'hostname'} eq "")
	{
		$tags{'hostname'} = taghelpers::ensureTag($sparser,"JAIL.Hostname");
	}
	$tags{'hostname'} = lc($tags{'hostname'});
        $tags{'hostname'} =~ s/^\s+|\s+$//g ; # remove leading and trailing spaces
	if ($tags{'hostname'} eq "")
	{
		print(STDERR "Error in ".$sparser->getFilename()."\n");
		print(STDERR "missing hostname\n");
		exit(1);
	}


	# IP einlesen und validieren
	$tags{'ip'} = taghelpers::getIP($sparser);
	if (!($tags{'ip'} =~ /^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+$/))
	{
		print(STDERR "Error in ".$sparser->getFilename()."\n");
		print(STDERR "invalid IP: ".$tags{'ip'}."\n");
		exit(1);
	}
	
	$tags{'contactgroups'} = taghelpers::getContactgroups($sparser);
        $tags{'contactgroups'} =~ s/^\s+|\s+$//g ; # remove leading and trailing spaces
	if ($tags{'contactgroups'} eq "")
	{
		$tags{'contactgroups'} = "lfnet-support";
	}


	($tags{'description'},$tags{'shortdescription'}) = 
		taghelpers::getDescriptions($sparser);
	$tags{'vservhost'} = lc taghelpers::ensureTag($sparser,"JAIL.Jailserver");
	# Falls es ein VHost ist:
	if ($tags{'funktion'} =~ "vhost")
	{	
		$tags{'vservhost'} = $tags{'hostname'}
	}

	$tags{'vservtype'} = taghelpers::ensureTag($sparser,"JAIL.Type");
	$tags{'rack'} = taghelpers::getRack($sparser);

	$tags{'filename'} = $sparser->getFilename();
	chomp($tags{'filename'});

	# Hostgroups aus dem 'Check.mon.hostgroup' Tag und dem Jailserver
	# Tag zusammensetzen:
	#
	# ... fuer Jails:
	$tags{'hostgroups'} = taghelpers::ensureTag(
	   	$sparser,"check.mon.hostgroups");
	if ($tags{'vservhost'} ne "")
	{
		$tags{'hostgroups'} .= "," if ($tags{'hostgroups'} ne "");
		$tags{'hostgroups'} .= "vserv-".$tags{'vservhost'};
	}

	# Pruefen, ob mindestens ein "24x7_" Prefix zu finden ist; falls ja,
	# dann eine Mitgliedschaft erzeugen:
	if (($checkmon =~ /24x7_/) || ($checkmon =~ /24x7rz_/))
	{
		$tags{'hostgroups'} .= "," if ($tags{'hostgroups'} ne "");
		$tags{'hostgroups'} .= "lfnet-24x7";
	}

        # Das Kundenverzeichnis wird zum Namen einer Hostgroup
        $tags{'hostgroups'} .= "," if ($tags{'hostgroups'} ne "");
        $tags{'hostgroups'} .= 'customer-' . (split('/', $tags{'filename'}))[4];

	$tags{'hostgroups'} = lc($tags{'hostgroups'});

	# iLO-Eintraege einlesen
	$tags{'ilousername'} = taghelpers::ensureTag($sparser,"iLO.Username");
	$tags{'ilopassword'} = taghelpers::ensureTag($sparser,"iLO.Password");
	$tags{'iloip'} = taghelpers::ensureTag($sparser,"iLO.IP");
	if ($tags{'iloip'} and ! $tags{'iloip'} =~ /^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+$/)
	{
		print(STDERR "Warning in ".$sparser->getFilename()."\n");
		print(STDERR "invalid iLO.IP: ".$tags{'iloip'}."\n");
		$tags{'iloip'} = "";
	}

	return %tags;
}

sub generateNagConfDef
{
	my ($self,%tags) = @_;
	my $env = $tags{"env"};
	my $s;

	# Falls keine Checks generiert werden sollen, gleich beenden:
	my $checkmon = $tags{'check.mon'};
        $checkmon =~ s/^\s+|\s+$//g ; # remove leading and trailing spaces
	if (($checkmon eq "") || ($checkmon eq "no") || 
		($checkmon eq "none") || ($checkmon eq "manual"))
	{
		return "";
	}

        # Ob ein Hostname schon mal bearbeitet wurde, ist im
        # env Hash gespeichert. Dies pruefen...
        my $identifier = "host_".$tags{'hostname'};
        if (!defined $env->{$identifier})
        {
                # und in den env Hash speichern
                $env->{$identifier} = $tags{'filename'};
        }
        else
        {
		print(STDERR "Error in ".$tags{'filename'}."\n");
		print(STDERR "duplication: hostname was already defined in '");
		print(STDERR $env->{$identifier} . "'\n");
		exit(1);
        }

	# Wenn dieser Host einen VSERV Host hat, dann muss ggfls
	# eine Hostgroup erzeugt werden:
	if ($tags{'vservhost'} ne "")
	{
		# Ob die Hostgruppe schon mal generiert wurde, ist im
		# env Hash gespeichert. Dies pruefen...
		my $hostgroup = "hostgroup_".$tags{'vservhost'};
		if (!defined $env->{$hostgroup})
		{
			# und in den env Hash speichern
			$env->{$hostgroup} = 1;

			# Hostgroup Daten generieren:
			$s .= "# VSERV -> generiere Hostgroup: vserv-".$tags{'vservhost'}."\n";
			$s .= "define hostgroup {\n";
			$s .= "  hostgroup_name vserv-".$tags{'vservhost'}."\n";
			$s .= "  alias VSERVs auf $tags{'vservhost'}\n";
			$s .= "}\n\n";

			# Den VZ Host selbst auch in seine Hostgroup aufnehmen
			# Damit braucht man beim ACKen nur noch die Hostgroup
			# ACKen...
			$tags{'hostgroups'} .= ",vserv-".$tags{'vservhost'};
		}
	}

        # Ob eine Kunden-Hostgroup schon mal bearbeitet wurde, ist im
        # env Hash gespeichert. Dies pruefen...
        my $customername = (split('/', $tags{'filename'}))[4];
        my $hostgroupname = lc($customername);
        $identifier = "hostgroup_".$hostgroupname;
        if (!defined $env->{$identifier})
        {
                # und in den env Hash speichern
                $env->{$identifier} = 1;
                # Hostgroup Daten generieren:
                $s .= "# generiere allgemeine Hostgroup: customer-".$hostgroupname."\n";
                $s .= "define hostgroup {\n";
                $s .= "  hostgroup_name customer-".$hostgroupname."\n";
                $s .= "  alias Systeme von $customername\n";
                $s .= "}\n\n";
        }

        if ($tags{'contactgroups'} eq "lfnet-support") {
            # Ob eine kundenweite contactgroup schon mal bearbeitet wurde, ist im
            # env Hash gespeichert. Dies pruefen...
            my @cpath = split('/', $tags{'filename'});
            my $clog = join('/', @cpath[0..4]) . '/Log';
            $identifier = "contactgroup_".$clog;
            my $ccontactgroups = '';
            # use exists instead of defined as otherwise empty values cause trouble
            if (!exists $env->{$identifier})
            {
                my $cparser = TransLog::SimpleTagParser->new($clog);
                $ccontactgroups = taghelpers::getContactgroups($cparser);
                $ccontactgroups =~ s/^\s+|\s+$//g ; # remove leading and trailing spaces
                # und in den env Hash speichern
                $env->{$identifier} = $ccontactgroups;
            }
            else
            {
                $ccontactgroups = $env->{$identifier};
            }
            $tags{'contactgroups'} = "$ccontactgroups" if $ccontactgroups;
        }

	# Host Config Abschnitt zusammenstellen
	$s .= "# verarbeite Path: ".$tags{'filename'}."\n";
	$s .= "#active: $tags{'hostname'} $tags{'ip'} $tags{'check.mon.mk'}\n";
	$s .= "define host {\n";
        if ($tags{'check.mon.mk'} =~ /(ssh-hop|fromremote)/) {
            $s .= "  use failtolerant-host\n";
        } else {
            $s .= "  use generic-host\n";
        }
	$s .= "  notes_url http://www.office.lf.net".$tags{'filename'}."\n";
	$s .= "  host_name $tags{'hostname'}\n";
        if ($tags{'description'}) {
            $s .= "  alias ".$tags{'description'}."\n";
        } elsif ($tags{'shortdescription'}) {
            $s .= "  alias ".$tags{'shortdescription'}."\n";
        } else {
            $s .= "  alias ".$tags{'hostname'}."\n";
        }
	$s .= "  address $tags{'ip'}\n";
        if ($tags{'check.mon.mk'} =~ /ssh-hop/) {
            $s .= "  check_command check-mk-cache\n";
        } elsif ($tags{'check.mon.mk'} =~ /fromremote/) {
            $s .= "  check_command check-mk-hostfileage\n";
        }

	my $cgext = "";
	if ($checkmon =~ /24x7_/)
	{
		$cgext = ",lfnet-24x7";
		$s .= "  icon_image 24x7.jpg\n";
	}
	if ($checkmon =~ /24x7rz_/)
	{
		$cgext = ",lfnet-24x7-rz";
		$s .= "  icon_image 24x7.jpg\n";
	}
	$s .= "  contact_groups $tags{'contactgroups'}$cgext\n";

	# Hostgroups nur ausgeben, wenn nicht leer
	if ($tags{'hostgroups'} ne "")
	{
		$s .= "  hostgroups ".$tags{'hostgroups'}."\n";
	}
	$s .= "  _description $tags{'description'}\n";
	$s .= "  _shortdescription $tags{'shortdescription'}\n";
	$s .= "  _rack $tags{'rack'}\n";
	$s .= "  _vservhost $tags{'vservhost'}\n";
	$s .= "}\n\n";

	# Service-Templates zur Verwendung durch die einzelnen Module
	# und durch andere Datenquellen
	# -> per Default alle Services mit gleicher Contactgroup per Vererbung
	$s .= "define service {\n";
	$s .= "  use generic-service\n";
	$s .= "  name generic-service-$tags{'hostname'}\n";
	$s .= "  host_name $tags{'hostname'}\n";
	$s .= "  contact_groups $tags{'contactgroups'}\n";
	$s .= "  register 0\n";
	$s .= "}\n\n";

	$s .= "define service {\n";
	$s .= "  use service-with-stats\n";
	$s .= "  name service-with-stats-$tags{'hostname'}\n";
	$s .= "  host_name $tags{'hostname'}\n";
	$s .= "  contact_groups $tags{'contactgroups'}\n";
	$s .= "  register 0\n";
	$s .= "}\n\n";

        # ebenso die Contactgroup check_mk bekannt machen
        my @contactgroups = split(',', $tags{'contactgroups'});
        # lfnet-support wird bereits vom Basis-Template vererbt
        @contactgroups = grep { $_ ne 'lfnet-support' } @contactgroups;
        if (@contactgroups) {
            $s .= "#MK: service_contactgroups += [\n";
            foreach my $group (@contactgroups) {
                # TODO: remove whitespace? check_mk seems to cope just fine ...
                $s .= "#MK:     ( '$group', [ '$tags{'hostname'}' ], ALL_SERVICES ),\n";
            }
            $s .= "#MK: ]\n\n";
        }

        # etwaig vorhandene monitoring.{cfg,mk} importieren
        my @cfgpath = split('/', $tags{'filename'});
        my $cfgfile = join('/', @cfgpath[0..6]) . '/monitoring.cfg';
        if (-s $cfgfile) {
            $s .= "# importing from $cfgfile\n";
            open(CFGFILE, $cfgfile);
            while(<CFGFILE>) {
                chomp;
                if ($_ =~ /define\s+(command|host|timeperiod)\s*\{/) {
                    print(STDERR "ERROR: entry '$_' not allowed in '$cfgfile'\n");
                    exit(1);
                }
                $s .= "$_\n";
            }
            close(CFGFILE);
            $s .= "# end of import from $cfgfile\n\n";
        }
        $cfgfile = join('/', @cfgpath[0..6]) . '/monitoring.mk';
        if (-s $cfgfile) {
            $s .= "#MK: # importing from $cfgfile\n";
            open(CFGFILE, $cfgfile);
            while(<CFGFILE>) {
                chomp;
                if ($_ =~ /all_hosts/) {
                    print(STDERR "ERROR: entry '$_' not allowed in '$cfgfile'\n");
                    exit(1);
                }
                $s .= "#MK: $_\n";
            }
            close(CFGFILE);
            $s .= "#MK: # end of import from $cfgfile\n\n";
        }

        # Ob eine kundenweite monitoring.cfg schon mal bearbeitet wurde, ist im
        # env Hash gespeichert. Dies pruefen...
        $cfgfile = join('/', @cfgpath[0..4]) . '/monitoring.cfg';
        $identifier = "cfg_".$cfgfile;
        if (!defined $env->{$identifier})
        {
            if (-s $cfgfile) {
                $s .= "# importing from $cfgfile\n";
                open(CFGFILE, $cfgfile);
                while(<CFGFILE>) {
                    chomp;
                    if ($_ =~ /define\s+(command|host|timeperiod)\s*\{/) {
                        print(STDERR "ERROR: entry '$_' not allowed in '$cfgfile'\n");
                        exit(1);
                    }
                    $s .= "$_\n";
                }
                close(CFGFILE);
                $s .= "# end of import from $cfgfile\n\n";
            }
            # und in den env Hash speichern
            $env->{$identifier} = 1;
        }
        # Ob eine kundenweite monitoring.mk schon mal bearbeitet wurde, ist im
        # env Hash gespeichert. Dies pruefen...
        $cfgfile = join('/', @cfgpath[0..4]) . '/monitoring.mk';
        $identifier = "mk_".$cfgfile;
        if (!defined $env->{$identifier})
        {
            if (-s $cfgfile) {
                $s .= "#MK: # importing from $cfgfile\n";
                open(CFGFILE, $cfgfile);
                while(<CFGFILE>) {
                    chomp;
                    if ($_ =~ /all_hosts/) {
                        print(STDERR "ERROR: entry '$_' not allowed in '$cfgfile'\n");
                        exit(1);
                    }
                    $s .= "#MK: $_\n";
                }
                close(CFGFILE);
                $s .= "#MK: # end of import from $cfgfile\n\n";
            }
            # und in den env Hash speichern
            $env->{$identifier} = 1;
        }

	return $s;
}

sub isCheckMonEntryValid
{
	my ($self,$s) = @_;

	return 0;
}

1;
